@@ -59,6 +59,7 @@ typedef struct RmFmtTable {
     GQueue *handler_order;
     GRecMutex state_mtx;
     RmSession *session;
+    GTimeVal first_backup_timestamp;
 
     /* Group of RmFiles that will be cached until exit */
     GQueue groups;
@@ -262,6 +263,11 @@ bool rm_fmt_is_stream(RmFmtTable *self, RmFmtHandler *handler);
  */
 bool rm_fmt_has_formatter(RmFmtTable *self, const char *name);
 
+/**
+ * @brief Remove all registered formatters with `name`.
+ */
+void rm_fmt_remove_by_name(RmFmtTable *self, char *name);
+
 /**
  * You can use this template for implementing new RmFmtHandlers.
  * All callbacks are not required to be implemented, leave them to NULL if
