@@ -46,23 +46,33 @@ static void rm_fmt_head(_UNUSED RmSession *session, _UNUSED RmFmtHandler *parent
         return;
     }
 
-    fprintf(out, "%s%s%s%s%s%s%s\n", "type", CSV_SEP, "path", CSV_SEP, "size", CSV_SEP,
-            "checksum");
+    fprintf(
+        out,
+        "%s%s%s%s%s%s%s\n",
+        "type", CSV_SEP,
+        "path", CSV_SEP,
+        "size", CSV_SEP,
+        "checksum"
+    );
 }
 
 static void rm_fmt_elem(_UNUSED RmSession *session, _UNUSED RmFmtHandler *parent,
                         FILE *out, RmFile *file) {
-    if(file->lint_type == RM_LINT_TYPE_UNIQUE_FILE &&
-       (!file->digest || !session->cfg->write_unfinished)) {
-        /* unique file with no partial checksum */
-        return;
+    if(file->lint_type == RM_LINT_TYPE_UNIQUE_FILE) {
+        if(!rm_fmt_get_config_value(session->formats, "csv", "unique")) {
+            if(!file->digest || !session->cfg->write_unfinished) {
+                return;
+            }
+        }
     }
 
-    char checksum_str[rm_digest_get_bytes(file->digest) * 2 + 1];
-    memset(checksum_str, '0', sizeof(checksum_str));
-    checksum_str[sizeof(checksum_str) - 1] = 0;
+    char *checksum_str = NULL;
+    size_t checksum_size = 0;
 
-    if(file->digest) {
+    if(file->digest != NULL) {
+        checksum_size = rm_digest_get_bytes(file->digest) * 2 + 1;
+        checksum_str = g_slice_alloc0(checksum_size);
+        checksum_str[checksum_size - 1] = 0;
         rm_digest_hexstring(file->digest, checksum_str);
     }
 
@@ -70,10 +80,20 @@ static void rm_fmt_elem(_UNUSED RmSession *session, _UNUSED RmFmtHandler *parent
     RM_DEFINE_PATH(file);
     char *clean_path = rm_util_strsub(file_path, CSV_QUOTE, CSV_QUOTE "" CSV_QUOTE);
 
-    fprintf(out, CSV_FORMAT, rm_file_lint_type_to_string(file->lint_type), clean_path,
-            file->actual_file_size, checksum_str);
+    fprintf(
+        out,
+        CSV_FORMAT,
+        rm_file_lint_type_to_string(file->lint_type),
+        clean_path,
+        file->actual_file_size,
+        checksum_str
+    );
 
     g_free(clean_path);
+
+    if(checksum_str != NULL) {
+        g_slice_free1(checksum_size, checksum_str);
+    }
 }
 
 static RmFmtHandlerProgress CSV_HANDLER_IMPL = {
@@ -85,7 +105,7 @@ static RmFmtHandlerProgress CSV_HANDLER_IMPL = {
         .elem = rm_fmt_elem,
         .prog = NULL,
         .foot = NULL,
-        .valid_keys = {"no_header", NULL},
+        .valid_keys = {"no_header", "unique", NULL},
     }};
 
 RmFmtHandler *CSV_HANDLER = (RmFmtHandler *)&CSV_HANDLER_IMPL;
