@@ -10,7 +10,7 @@ def test_simple():
     create_file('xxx', '2/a')
     create_file('xxx', 'a')
 
-    head, *data, footer = run_rmlint('-pp -D --rank-by A')
+    head, *data, footer = run_rmlint('-p -D --rank-by A')
 
     assert 2 == sum(find['type'] == 'duplicate_dir' for find in data)
 
@@ -32,7 +32,7 @@ def test_diff():
     create_file('xxx', '2/a')
     create_file('xxx', '3/a')
     create_file('yyy', '3/b')
-    head, *data, footer = run_rmlint('-pp -D --rank-by A')
+    head, *data, footer = run_rmlint('-p -D --rank-by A')
 
     assert 2 == sum(find['type'] == 'duplicate_dir' for find in data)
     assert data[0]['size'] == 3
@@ -49,7 +49,7 @@ def test_same_but_not_dupe():
     create_file('xxx', '1/a')
     create_file('xxx', '2/a')
     create_file('xxx', '2/b')
-    head, *data, footer = run_rmlint('-pp -D --rank-by A')
+    head, *data, footer = run_rmlint('-p -D --rank-by A')
 
     # No duplicate dirs, but 3 duplicate files should be found.
     assert 0 == sum(find['type'] == 'duplicate_dir' for find in data)
@@ -64,7 +64,7 @@ def test_hardlinks():
     create_link('2/a', '2/link1')
     create_link('2/a', '2/link2')
 
-    head, *data, footer = run_rmlint('-pp -D -l -S a')
+    head, *data, footer = run_rmlint('-p -D -l -S a')
     assert len(data) is 5
     assert data[0]['type'] == 'duplicate_dir'
     assert data[0]['path'].endswith('1')
@@ -109,7 +109,7 @@ def test_deep_simple():
     create_file('xxx', 'd/b/empty')
     create_file('xxx', 'd/a/1')
     create_file('xxx', 'd/b/empty')
-    head, *data, footer = run_rmlint('-pp -D -S a')
+    head, *data, footer = run_rmlint('-p -D -S a')
 
     assert data[0]['path'].endswith('d/a')
     assert data[1]['path'].endswith('d/b')
@@ -120,7 +120,7 @@ def test_deep_simple():
 def test_dirs_with_empty_files_only():
     create_file('', 'a/empty')
     create_file('', 'b/empty')
-    head, *data, footer = run_rmlint('-pp -D -S a -T df,dd --size 0')
+    head, *data, footer = run_rmlint('-p -D -S a -T df,dd --size 0')
 
     assert len(data) == 2
     assert data[0]['path'].endswith('a')
@@ -128,10 +128,10 @@ def test_dirs_with_empty_files_only():
     assert data[1]['path'].endswith('b')
     assert data[1]['type'] == "duplicate_dir"
 
-    head, *data, footer = run_rmlint('-pp -D -S a -T df,dd')
+    head, *data, footer = run_rmlint('-p -D -S a -T df,dd')
     assert len(data) == 0
 
-    head, *data, footer = run_rmlint('-pp -D -S a --size 0')
+    head, *data, footer = run_rmlint('-p -D -S a --size 0')
     assert len(data) == 2
 
     data.sort(key=lambda elem: elem["path"])
@@ -156,7 +156,7 @@ def test_deep_full():
 
     # subprocess.call('tree ' + TESTDIR_NAME, shell=True)
     # subprocess.call('./rmlint -p -S a -D ' + TESTDIR_NAME, shell=True)
-    head, *data, footer = run_rmlint('-pp -D -S a')
+    head, *data, footer = run_rmlint('-p -D -S a')
 
     assert len(data) == 6
 
@@ -225,7 +225,7 @@ def test_symlinks():
     create_file('xxx', 'b/z')
     create_link('b/z', 'b/x', symlink=True)
 
-    head, *data, footer = run_rmlint('-pp -D -S a -F')
+    head, *data, footer = run_rmlint('-p -D -S a -F')
 
     assert len(data) == 2
     assert data[0]['path'].endswith('z')
@@ -233,22 +233,14 @@ def test_symlinks():
     assert data[1]['path'].endswith('z')
     assert not data[1]['is_original']
 
-    head, *data, footer = run_rmlint('-pp -D -S a -f')
+    head, *data, footer = run_rmlint('-p -D -S a -f')
 
-    assert len(data) == 4
+    assert len(data) == 2
     assert data[0]['path'].endswith('/a')
     assert data[0]['is_original']
     assert data[1]['path'].endswith('/b')
     assert not data[1]['is_original']
 
-    # z must come first, since it's the physical real file
-    assert data[2]['path'].endswith('/a/z')
-    assert data[2]['is_original']
-
-    # x is a duplicate of z when following links
-    assert data[3]['path'].endswith('/a/x')
-    assert not data[3]['is_original']
-
 
 def mount_bind_teardown_func():
     if runs_as_root():
@@ -425,7 +417,7 @@ def test_equal_content_different_layout():
     create_file('xxx', "tree-b/x")
     create_file('yyy', "tree-b/y")
 
-    head, *data, footer = run_rmlint('-pp -D --rank-by a')
+    head, *data, footer = run_rmlint('-p -D --rank-by a')
 
     assert data[0]["path"].endswith("tree-a")
     assert data[0]["is_original"] is True
@@ -433,7 +425,7 @@ def test_equal_content_different_layout():
     assert data[1]["is_original"] is False
 
     # Now, try to honour the layout
-    head, *data, footer = run_rmlint('-pp -Dj --rank-by a')
+    head, *data, footer = run_rmlint('-p -Dj --rank-by a')
     for point in data:
         assert point["type"] == "duplicate_file"
 
